<?php
################################################################################
### This disclaimer must be kept intact in order to use this product.        ###
################################################################################
### Project: jT CarFramework [http://intersofts.com]
### Author: J. Toshio Taniguchi
### Since: 27.04.2011
### Version: 1.0.0
### Copyrights: J. Toshio Taniguchi
### Contact: j.taniguchi@taniguchi-blog.com
################################################################################
### CONTRIBUTORS
### - none
################################################################################
class adminupdateinstaller {
    private $lang;
    private $form_message;
    private $form_status;
    private $form_style;
    private $company_list;
    private $package_installation;
    private $installation_status = false;
    private $package_compilation;
    
    /**
     * Constructor
     * <br>---------------------------------------------------------------------
     */
    public function adminupdateinstaller($lang=array()){
        $this->lang = $lang;
                
        // handle update installer form.
        if( isset($_POST['pckbtn']) ){
            $this->updateFrameworkInstaller();
        }
        
        // handle cancel installation.
        if( isset($_POST['cancelbtn']) ){
            $this->handleCancelInstall();
        }
        
        // handle install form.
        if( isset($_POST['installbtn']) ){
            $this->handleInstall();
        }
    }
    
    /**
     * Private method: handle package installation
     * <br>---------------------------------------------------------------------
     * @return void
     */
    private function handleInstall(){
        /*
         * the goal is to take the files located on the package and place it on
         * the framework files. this should be used with extreme caution...
         * why allow files to be replaced? - because of core updates and new
         * features which will have to have access to those areas...if you have
         * a better idea how to do this, please, share!
         */
        
        // validate session.
        if( !isset($_SESSION['update_package']) ){
            // something went wrong...session not set???
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to install update package. Data not set. Error throw at ' . __FILE__ . ':' . __LINE__);
            
            $this->form_message = $this->lang['AUPDATER_PACK_INSTALL_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            
            return;
        }
        
        // make sure that package data exists.
        $files = @$_SESSION['update_package']['data'];
        $detail = @$_SESSION['update_package']['details'];
        
        if( !is_array($files) || !is_array($detail) ){
            // something went wrong...data is not set...
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to install update package. Data not set. Error throw at ' . __FILE__ . ':' . __LINE__);
            
            $this->form_message = $this->lang['AUPDATER_PACK_INSTALL_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            
            return;
        }
        
        // get update package url.
        $pack_url = @$detail['url'];
        
        if( !file_exists($pack_url) ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Package URL does not exists. Package must be manually removed from tmp dir. Error throw at ' . __FILE__ . ':' . __LINE__);
            
            $this->form_message = $this->lang['AUPDATER_PACK_INSTALL_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            
            return;
        }
        
        // update each file declared.
        $complete = true;
        for( $i=0; $i < count($files); $i++ ){
            // get filename.
            $fname = explode('/', $files[$i]['filename']);
            $fname = $fname[(count($fname)-1)];
            
            // move file to it's destination.
            $dest = BASE_ROOT . $files[$i]['destination'] . $fname;
            $file = $pack_url . $files[$i]['filename'];
            
			if( !file_exists(BASE_ROOT . $files[$i]['destination']) ){
				if( !@mkdir(BASE_ROOT . $files[$i]['destination'],0777) ){
					$complete = false;
					require_once BASE_CLASS . 'class-log.php';
					
					LogReport::write('Unable create directory ['.BASE_ROOT . $files[$i]['destination'].'] ' . __FILE__ . ':' . __LINE__);
				}
			}
			
            if( !@copy($file, $dest) ){
                $complete = false;
                break;
            }
        }
        
        // update failed.
        if( !$complete ){
            require_once BASE_CLASS . 'class-log.php';
            require_once BASE_CLASS . 'class-utilities.php';
            
            LogReport::write('Unable install update due an error at ' . BASE_ROOT . $files[$i]['destination'] . ' : '  . __FILE__ . ':' . __LINE__);
            Utilities::remove_directory($pack_url);
            
            $this->form_message = $this->lang['AUPDATER_PACK_INSTALL_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            
            return;
        }
        
        unset($_SESSION['update_package']);
        
        require_once BASE_CLASS . 'class-utilities.php';
        
        Utilities::remove_directory($detail['url']);
        
        $this->form_message = $this->lang['AUPDATER_PACK_INSTALL_SUCCESS'];
        $this->form_status = true;
        $this->form_style = 'alert-success';

        return;
    }
    
    /**
     * Private method: cancel update installation
     * <br>---------------------------------------------------------------------
     * @return void
     */
    private function handleCancelInstall(){
        // remove installation package from temp dir.
        if( !isset($_POST['pck']) ){
            return;
        }
        
        $pck = trim($_POST['pck']);
        $pck = base64_decode($pck);
        
        if( !file_exists($pck) ){
            return;
        }
                
        require_once BASE_CLASS . 'class-utilities.php';
        
        Utilities::remove_directory($pck);
        
        $this->form_message = $this->lang['AUPDATER_CANCEL_INSTALL'];
        $this->form_status = true;
        $this->form_style = 'alert-success';
        
        if( isset($_SESSION['update_package']) ){
            unset($_SESSION['update_package']);
        }
        
        return;
    }
    
    /**
     * Private method: update framework handler
     * <br>---------------------------------------------------------------------
     * @return void
     */
    private function updateFrameworkInstaller(){
        $fname = $_FILES['pckInput']['name'];
        $fsize = $_FILES['pckInput']['size'];
        
        // validate package size.
        if( $fsize < 1 ){
            $this->form_message = $this->lang['AUPDATER_INVALID_PACKAGE_SIZE'] . $this->getMaxUploadFilesize();
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        // validate extension.
        $fname = explode('.',$fname);
        $fname = $fname[(count($fname)-1)];
        $fname = strtolower($fname);
        
        if( $fname != 'zip' ){
            $this->form_message = $this->lang['AUPDATER_INVALID_PACKAGE_FORMAT'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        // define upload file's path
        $fname = date('YmdHis');
        $path = BASE_ROOT . 'tmp/' . $fname . '.zip';
        
        // upload package to tmp dir.
        if( !@move_uploaded_file($_FILES['pckInput']['tmp_name'], $path) ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to upload update package due an error at ' . __FILE__ . ':' . __LINE__);
            
            $this->form_message = $this->lang['AUPDATER_UPLOAD_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        // uncompress zip file to temp. dir.
        $zip = new ZipArchive();
        
        if( !$files = $zip->open($path) ){
            $this->form_message = $this->lang['AUPDATER_UNPACK_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        $extract_to = BASE_ROOT . '/tmp/' . $fname . '/';
        
        if( !$zip->extractTo($extract_to) ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to extraxt update package due an error at ' . __FILE__ . ':' . __LINE__);
            
            $zip->close();
            $this->form_message = $this->lang['AUPDATER_UNPACK_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        $zip->close();
        
        // remove uploaded package. 
        // note: because windows SUCKS, the uploaded file
        // may not be removed because it comes from the system's Temp directory
        // which has a different owner than the localhost server. if you are 
        // using windows, learn how to fix the permission problem or just use
        // linux for your server.
        @unlink($path);
        
        unset($zip);
        
        // now that we have it unzip, check for the XML main file. it MUST be
        // called package.xml
        if( !$xml = @simplexml_load_file(BASE_ROOT . 'tmp/' . $fname . '/package.xml') ){
            require_once BASE_CLASS . 'class-log.php';
            
            LogReport::write('Unable to load XML from packaging file. XML corrupted or non existent. Error throw at ' . __FILE__ . ':' . __LINE__);
            
            $this->form_message = $this->lang['AUPDATER_UNPACK_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-warning';
            
            return;
        }
        
        // get info from package file.
        $author_name  = @(string)$xml->compilation->author;
        $author_web   = @(string)$xml->compilation->author_web;
        $comp_date    = @(string)$xml->compilation->date;
        $comp_version = @(string)$xml->compilation->version;
        $description  = @(string)$xml->compilation->description;
        $package_logo = @(string)$xml->compilation->package_logo;
        $package_files= array();
        
        $this->package_compilation = array(
            'url' => BASE_ROOT . 'tmp/' . $fname . '/',
            'author' => $author_name,
            'author_web' => $author_web,
            'date' => $comp_date,
            'version' => $comp_version,
            'description' => $description,
            'logo' => 'tmp/' . $fname . '/' . $package_logo
        );
                
        for( $i=0; $i < count($xml->package->file); $i++ ){
            $file = @$xml->package->file[$i];
            
            $obj = array(
                'filename' => (string)$file->filename,
                'destination' => (string)$file->destination
            );
            
            array_push($package_files, $obj);
            
            unset($file);
        }
        
        // check for author, data and version. if empty, package is not valid.
        // if so, remove uploaded directory.
        if( empty($author_name) || empty($comp_date) || empty($comp_version) ){
            require_once BASE_CLASS . 'class-utilities.php';
            
            Utilities::remove_directory(BASE_ROOT . 'tmp/' . $fname . '/');
            
            $this->form_message = $this->lang['AUPDATER_UNPACK_ERROR'];
            $this->form_status = true;
            $this->form_style = 'alert-error';
            return;
        }
        
        $this->package_installation = $package_files;
        
        unset($package_files);
        unset($fname);
        
        // set installation status which will be used to confirm the installation
        // to the user.
        $this->installation_status = true;
        
        // set everything to session so we can get it's data after the page
        // refresh.
        if( isset($_SESSION['update_package']) ){
            unset($_SESSION['update_package']);
        }
        
        $_SESSION['update_package'] = array(
            'details' => $this->package_compilation,
            'data' => $this->package_installation
        );
    }
    
    /**
     * Public method: get installation status
     * <br>---------------------------------------------------------------------
     * @return boolean
     */
    public function getInstallationStatus(){
        return $this->installation_status;
    }
    
    /**
     * Public method: get installation details
     * <br>---------------------------------------------------------------------
     * @return array
     */
    public function getPackageDetails(){
        return $this->package_compilation;
    }
    
    /**
     * Public method: get max server upload filesize
     * <br>---------------------------------------------------------------------
     * @return string
     */
    public function getMaxUploadFilesize(){
        $max_upload = (int)(ini_get('upload_max_filesize'));
        $max_post = (int)(ini_get('post_max_size'));
        $memory_limit = (int)(ini_get('memory_limit'));
        return min($max_upload, $max_post, $memory_limit) . 'MB';
    }
    
    /**
     * Public method: get form status
     * <br>---------------------------------------------------------------------
     * @return bool.
     */
    public function getFormStatus()
    {
        return $this->form_status;
    }

    /**
     * Public method: get form message 
     * <br>---------------------------------------------------------------------
     * @return string.
     */
    public function getFormMessage()
    {
        return $this->form_message;
    }

    /**
     * Public method: get form style
     * <br>---------------------------------------------------------------------
     * @return string.
     */
    public function getFormStyle()
    {
        return $this->form_style;
    }
}